<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:doml="http://dods.enhydra.org/schema/eight"
	xmlns="http://dods.enhydra.org/schema/eight"
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">

<xsl:output method="xml"
            encoding="utf-8"
            indent="yes"/>

    <!-- ################################################################# -->
    <!-- 	Transforms old DOML schema files into new DOML schema files.								-->
    <!-- 	created by Dejan Betos															-->
    <!-- ################################################################# -->

<!--
    <xsl:template match="*">
				<xsl:copy-of select="."/>
	  </xsl:template>-->

<!-- ***************************************************** main templates ***************************************************** -->

    <xsl:template match="doml|Doml|DOML">
		 <doml:doml
		    id="doml8-model"
		    name="DODS Project"
		    xmlns:doml="http://dods.enhydra.org/schema/eight"
		    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		    xsi:schemaLocation="http://dods.enhydra.org/schema/eight domlEight.xsd">

			<xsl:if test="child::*[name(.)='projectname']">
				<xsl:attribute name="name">
					<xsl:value-of select="projectname"/>
				</xsl:attribute>
			</xsl:if>
			
			<xsl:if test="child::*[name(.)='author']">				
				<xsl:attribute name="author">
					<xsl:value-of select="author"/>
				</xsl:attribute>
			</xsl:if>

			    <doml:defaults>
			        <doml:extendedAttributes>
			        	<doml:extendedAttribute id="defaultCacheSize"
			        		name="Cache.MaxSize" val="0">
			        	</doml:extendedAttribute>
			        	<doml:extendedAttribute id="defaultSimpleQueryCacheSize" name="Cache.SimpleQuerySize" val="0"></doml:extendedAttribute>
			        	<doml:extendedAttribute id="defaultComplexQueryCacheSize"
			        		name="Cache.ComplexQuerySize" val="0">
			        	</doml:extendedAttribute>
			        </doml:extendedAttributes>
			        <doml:concurrency
			            column="Version"
			            id="version"
			            method="increment">
			            <description>
			                Default concurency mechanism would be appended to all
			                tables in model, if not overriden by lower level
			                elements.
			            </description>
			        </doml:concurrency>
			        <doml:primaryKey id="ObjectId">
			            <description>
			                Default primary key to be applied to all tables in doml.
			            </description>
			            <doml:column
			                canBeNull="false"
			                id="OId"
			                logicalTypeId="BigDecimal19">
			                <doml:value
			                    access="read-only"
			                    auto="generated"
			                    formula="ObjectId" />
			            </doml:column>
			        </doml:primaryKey>
			    </doml:defaults>
				
				<xsl:apply-templates/>


			<!-- created one dimension for test -->
				<doml:dimensions>
			        <doml:dimension id="dods-common">
			            <doml:logicalDomain id="BIT" baseDataTypeId="NUMERIC" precision="3" scale="0">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="BIT"/>
								<doml:extendedAttribute id="javaType" val="boolean"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="TINYINT" baseDataTypeId="NUMERIC" precision="3" scale="0">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="TINYINT"/>
								<doml:extendedAttribute id="javaType" val="byte"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="SMALLINT" baseDataTypeId="NUMERIC" precision="8" scale="0">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="SMALLINT"/>
								<doml:extendedAttribute id="javaType" val="short"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="INTEGER" baseDataTypeId="NUMERIC" precision="12" scale="0">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="INTEGER"/>
								<doml:extendedAttribute id="javaType" val="int"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="BIGINT" baseDataTypeId="NUMERIC" precision="20" scale="0">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="BIGINT"/>
								<doml:extendedAttribute id="javaType" val="long"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="REAL" baseDataTypeId="NUMERIC" precision="16" scale="8">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="REAL"/>
								<doml:extendedAttribute id="javaType" val="float"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="FLOAT" baseDataTypeId="NUMERIC" precision="16" scale="8">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="FLOAT"/>
								<doml:extendedAttribute id="javaType" val="float"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="DOUBLE" baseDataTypeId="NUMERIC" precision="16" scale="16">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="DOUBLE"/>
								<doml:extendedAttribute id="javaType" val="double"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="DECIMAL" baseDataTypeId="NUMERIC" precision="19" scale="0">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="DECIMAL"/>
								<doml:extendedAttribute id="javaType" val="java.math.BigDecimal"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="CHAR" baseDataTypeId="TEXTUAL" size="2">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="CHAR"/>
								<doml:extendedAttribute id="javaType" val="String"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="VARCHAR" baseDataTypeId="TEXTUAL" size="255">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="VARCHAR"/>
								<doml:extendedAttribute id="javaType" val="String"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
			            <doml:logicalDomain id="LONGVARCHAR" baseDataTypeId="TEXTUAL" size="2000">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="LONGVARCHAR"/>
								<doml:extendedAttribute id="javaType" val="String"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="VARBINARY" baseDataTypeId="BINARY">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="VARBINARY"/>
								<doml:extendedAttribute id="javaType" val="byte[]"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="LONGVARBINARY" baseDataTypeId="BINARY">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="LONGVARBINARY"/>
								<doml:extendedAttribute id="javaType" val="byte[]"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="DATE" baseDataTypeId="DATETIME">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="DATE"/>
								<doml:extendedAttribute id="javaType" val="Date"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="TIME" baseDataTypeId="DATETIME">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="TIME"/>
								<doml:extendedAttribute id="javaType" val="Time"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="TIMESTAMP" baseDataTypeId="DATETIME">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="TIMESTAMP"/>
								<doml:extendedAttribute id="javaType" val="Timestamp"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="NODE" baseDataTypeId="XML">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="NODE"/>
								<doml:extendedAttribute id="javaType" val="org.w3c.dom.Node"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						
						<doml:logicalDomain id="BigDecimal19" baseDataTypeId="NUMERIC" precision="19" scale="0">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="dbType" val="DECIMAL"/>
								<doml:extendedAttribute id="javaType" val="java.math.BigDecimal"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
						<doml:logicalDomain id="ShortInt" baseDataTypeId="NUMERIC" precision="8" scale="0" />
			            <doml:logicalDomain id="boolean" baseDataTypeId="NUMERIC" precision="1" scale="0" />
			            <doml:logicalDomain id="XML" baseDataTypeId="BINARY">
			                <doml:extendedAttributes>
			                    <doml:extendedAttribute id="MSSQL2005-Map-To-XML" val="true" />
								<doml:extendedAttribute id="dbType" val="LONGVARCHAR"/>
								<doml:extendedAttribute id="javaType" val="org.w3c.dom.Node"/>
			                </doml:extendedAttributes>
			            </doml:logicalDomain>
			        </doml:dimension>
				</doml:dimensions>
				
			

		 </doml:doml>
	</xsl:template>

	<xsl:template match="author|projectname">
		<!--do nothing-->
	</xsl:template>


	<xsl:template match="database|Database|DATABASE">
		<doml:databases>
	        <doml:database>
				<xsl:attribute name="id">
					<xsl:value-of select="@database"/>
				</xsl:attribute>
				<xsl:attribute name="name">
					<xsl:value-of select="@database"/>
				</xsl:attribute>
				<xsl:attribute name="owner">
					<xsl:value-of select="@templateset"/>
				</xsl:attribute>
				
				<xsl:if test="@database!='' or @generateSecure!='' or @generateInsecure!='' or @massDeletes!='' or @massUpdates!='' or @dirtyDOs!=''">
					<doml:extendedAttributes>
						<xsl:if test="@database!=''">
								<doml:extendedAttribute id="database">
								<xsl:attribute name="val">
									<xsl:value-of select="@database"/>
								</xsl:attribute>
								</doml:extendedAttribute>
						</xsl:if>
						<xsl:if test="@generateSecure!=''">
								<doml:extendedAttribute id="generateSecure">
								<xsl:attribute name="val">
									<xsl:value-of select="@generateSecure"/>
								</xsl:attribute>
								</doml:extendedAttribute>
						</xsl:if>
						<xsl:if test="@generateInsecure!=''">
								<doml:extendedAttribute id="generateInsecure">
								<xsl:attribute name="val">
									<xsl:value-of select="@generateInsecure"/>
								</xsl:attribute>
								</doml:extendedAttribute>
						</xsl:if>
						<xsl:if test="@massDeletes!=''">
								<doml:extendedAttribute id="massDeletes">
								<xsl:attribute name="val">
									<xsl:value-of select="@massDeletes"/>
								</xsl:attribute>
								</doml:extendedAttribute>
						</xsl:if>
						<xsl:if test="@massUpdates!=''">
								<doml:extendedAttribute id="massUpdates">
								<xsl:attribute name="val">
									<xsl:value-of select="@massUpdates"/>
								</xsl:attribute>
								</doml:extendedAttribute>
						</xsl:if>
						<xsl:if test="@dirtyDOs!=''">
								<doml:extendedAttribute id="dirtyDOs">
									<xsl:attribute name="val">
										<xsl:value-of select="@dirtyDOs"/>
									</xsl:attribute>
								</doml:extendedAttribute>
						</xsl:if>

					</doml:extendedAttributes>
				</xsl:if>
			
				<xsl:apply-templates/>
			</doml:database>
		</doml:databases>
	</xsl:template>

	
    <xsl:template match="package|Package|PACKAGE">
				<xsl:apply-templates/>
	</xsl:template>
	
	
    <xsl:template match="table|Table|TABLE">
		<doml:table>
			<xsl:attribute name="id">
				<xsl:variable name="fullId" select="@id"/>
				<xsl:call-template name="idShrunk">
					<xsl:with-param name="idList" select="$fullId"/>
				</xsl:call-template>		
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:choose>
					<xsl:when test="@dbTableName!=''">
						<xsl:value-of select="@dbTableName"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:variable name="fullId" select="@id"/>
						<xsl:call-template name="idShrunk">
							<xsl:with-param name="idList" select="$fullId"/>
						</xsl:call-template>		
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			
			<doml:extendedAttributes>
				<xsl:if test="@generateSecure!=''">
						<doml:extendedAttribute id="generateSecure">
							<xsl:attribute name="val">
								<xsl:value-of select="@generateSecure"/>
							</xsl:attribute>
						</doml:extendedAttribute>
				</xsl:if>
				<xsl:if test="@generateInsecure!=''">
						<doml:extendedAttribute id="generateInsecure">
							<xsl:attribute name="val">
								<xsl:value-of select="@generateInsecure"/>
							</xsl:attribute>
						</doml:extendedAttribute>
				</xsl:if>
				<xsl:if test="@massDeletes!=''">
						<doml:extendedAttribute id="massDeletes">
							<xsl:attribute name="val">
								<xsl:value-of select="@massDeletes"/>
							</xsl:attribute>
						</doml:extendedAttribute>
				</xsl:if>
				<xsl:if test="@massUpdates!=''">
						<doml:extendedAttribute id="massUpdates">
							<xsl:attribute name="val">
								<xsl:value-of select="@massUpdates"/>
							</xsl:attribute>
						</doml:extendedAttribute>
				</xsl:if>
				<xsl:if test="@multidb!=''">
						<doml:extendedAttribute id="multidb">
							<xsl:attribute name="val">
								<xsl:value-of select="@multidb"/>
							</xsl:attribute>
						</doml:extendedAttribute>
				</xsl:if>
				<xsl:if test="@dirtyDOs!=''">
						<doml:extendedAttribute id="dirtyDOs">
							<xsl:attribute name="val">
								<xsl:value-of select="@dirtyDOs"/>
							</xsl:attribute>
						</doml:extendedAttribute>
				</xsl:if>

				<doml:extendedAttribute id="javaName">
					<xsl:attribute name="val">
						<xsl:value-of select="@id"/>
					</xsl:attribute>
				</doml:extendedAttribute>
			</doml:extendedAttributes>
			<doml:columns>
			<xsl:for-each select="column">
				<xsl:apply-templates select="."/>
			</xsl:for-each>
			</doml:columns>
			<xsl:if test="index|statistics">
			<doml:indexes>
			<xsl:for-each select="index|statistics">
				<xsl:apply-templates select="."/>
			</xsl:for-each>
			</doml:indexes>
			</xsl:if>
				<!--<xsl:apply-templates/>-->
		</doml:table>
	</xsl:template>

   <xsl:template match="column|Column|COLUMN">
		<xsl:variable name="javaType">
			<xsl:variable name="fullId" select="type/@javaType"/>
			<xsl:call-template name="idShrunk">
				<xsl:with-param name="idList" select="$fullId"/>
			</xsl:call-template>		
		</xsl:variable>
		<xsl:choose>
		<xsl:when test="not(child::*[name(.)='referenceObject'])">
			<doml:column>
					<xsl:attribute name="id">
						<xsl:value-of select="@id"/>
					</xsl:attribute>
				
				<xsl:choose>
					<!-- dimension with logicalDomain "BigDecimal19" -->			
					<!--<xsl:when test="($javaType='BigDecimal' and type/@size='19,0') or $javaType='none'">-->
					<xsl:when test="($javaType='BigDecimal' and type/@size='19,0')">
						<xsl:attribute name="logicalTypeId">
							<xsl:value-of select="'BigDecimal19'"/>
						</xsl:attribute>
					</xsl:when>

					<!-- dimension with logicalDomain "LONGVARCHAR" and javatype "Node" is special case and is always XML in old doml files -->			
					<xsl:when test="$javaType='Node' and type/@dbType='LONGVARCHAR'">
						<xsl:attribute name="logicalTypeId">
							<xsl:value-of select="'XML'"/>
						</xsl:attribute>
					</xsl:when>

					<xsl:otherwise>
						<xsl:attribute name="logicalTypeId">
							<xsl:value-of select="type/@dbType"/>
						</xsl:attribute>
					</xsl:otherwise>
				</xsl:choose>

				<xsl:if test="type/@canBeNull='true'">
					<xsl:attribute name="canBeNull">
						<xsl:value-of select="type/@canBeNull"/>
					</xsl:attribute>
				</xsl:if>
				<xsl:if test="@usedForQuery!=''">
					<xsl:attribute name="usedForQuery">
						<xsl:value-of select="@usedForQuery"/>
					</xsl:attribute>
				</xsl:if>
<!--				<xsl:attribute name="logicalTypeId">
					<xsl:value-of select="type/@dbType"/>
				</xsl:attribute>-->
				<xsl:if test="type/@size!=''">
					<xsl:choose>
						<xsl:when test="contains(type/@size,',')">
							<xsl:attribute name="precision">
								<xsl:value-of select="substring-before(type/@size,',')"/>
							</xsl:attribute>
							<xsl:attribute name="scale">
								<xsl:value-of select="substring-after(type/@size,',')"/>
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="size">
								<xsl:value-of select="type/@size"/>
							</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:if>
<!--				<xsl:attribute name="id">
					<xsl:value-of select="@id"/>
				</xsl:attribute>-->
				
				<!--<xsl:if test="@generateSecure!='' or @generateInsecure!='' or type/@javaType!='' or child::*[name(.)='error'] or child::*[name(.)='javadoc']">-->
				<xsl:if test="@generateSecure!='' or @generateInsecure!='' or child::*[name(.)='error'] or child::*[name(.)='javadoc']
								or (type/@dbType='BIT' and type/@javaType!='boolean') or (type/@dbType='TINYINT' and type/@javaType!='byte') or 
								(type/@dbType='SMALLINT' and type/@javaType!='short') or (type/@dbType='INTEGER' and type/@javaType!='int') or 
								(type/@dbType='BIGINT' and type/@javaType!='long') or (type/@dbType='REAL' and type/@javaType!='float') or 
								(type/@dbType='FLOAT' and type/@javaType!='float') or (type/@dbType='DOUBLE' and type/@javaType!='double') or 
								(type/@dbType='DECIMAL' and type/@javaType!='java.math.BigDecimal') or (type/@dbType='INTEGER' and type/@javaType!='int') or 
								(type/@dbType='CHAR' and type/@javaType!='String') or (type/@dbType='VARCHAR' and type/@javaType!='String') or 
								(type/@dbType='LONGVARCHAR' and type/@javaType!='String') or (type/@dbType='VARBINARY' and type/@javaType!='byte[]') or 
								(type/@dbType='LONGVARBINARY' and type/@javaType!='byte[]') or (type/@dbType='DATE' and type/@javaType!='Date') or 
								(type/@dbType='TIME' and type/@javaType!='Time') or (type/@dbType='TIMESTAMP' and type/@javaType!='Timestamp') or 
								(type/@dbType='NODE' and type/@javaType!='org.w3c.dom.Node')">
					<doml:extendedAttributes>
						<xsl:if test="@generateSecure!=''">
								<doml:extendedAttribute id="generateSecure">
									<xsl:attribute name="val">
										<xsl:value-of select="@generateSecure"/>
									</xsl:attribute>
								</doml:extendedAttribute>
						</xsl:if>
						<xsl:if test="@generateInsecure!=''">
								<doml:extendedAttribute id="generateInsecure">
									<xsl:attribute name="val">
										<xsl:value-of select="@generateInsecure"/>
									</xsl:attribute>
								</doml:extendedAttribute>
						</xsl:if>
						<xsl:if test="(type/@dbType='BIT' and type/@javaType!='boolean') or (type/@dbType='TINYINT' and type/@javaType!='byte') or 
										(type/@dbType='SMALLINT' and type/@javaType!='short') or (type/@dbType='INTEGER' and type/@javaType!='int') or 
										(type/@dbType='BIGINT' and type/@javaType!='long') or (type/@dbType='REAL' and type/@javaType!='float') or 
										(type/@dbType='FLOAT' and type/@javaType!='float') or (type/@dbType='DOUBLE' and type/@javaType!='double') or 
										(type/@dbType='DECIMAL' and type/@javaType!='java.math.BigDecimal') or (type/@dbType='INTEGER' and type/@javaType!='int') or 
										(type/@dbType='CHAR' and type/@javaType!='String') or (type/@dbType='VARCHAR' and type/@javaType!='String') or 
										(type/@dbType='LONGVARCHAR' and type/@javaType!='String') or (type/@dbType='VARBINARY' and type/@javaType!='byte[]') or 
										(type/@dbType='LONGVARBINARY' and type/@javaType!='byte[]') or (type/@dbType='DATE' and type/@javaType!='Date') or 
										(type/@dbType='TIME' and type/@javaType!='Time') or (type/@dbType='TIMESTAMP' and type/@javaType!='Timestamp') or 
										(type/@dbType='NODE' and type/@javaType!='org.w3c.dom.Node')">
							<doml:extendedAttribute id="javaType">
								<xsl:attribute name="val">
									<xsl:value-of select="type/@javaType"/>
								</xsl:attribute>
							</doml:extendedAttribute>
						</xsl:if>
						
						<xsl:if test="child::*[name(.)='error']">
							<xsl:for-each select="error|Error|ERROR">
									<doml:extendedAttribute id="error">
										<doml:val><xsl:value-of select="."/></doml:val>
									</doml:extendedAttribute>
							</xsl:for-each>
						</xsl:if>
						<xsl:if test="child::*[name(.)='javadoc']">
							<xsl:for-each select="javadoc|Javadoc|JAVADOC">
									<doml:extendedAttribute id="javadoc">
										<doml:val><xsl:value-of select="."/></doml:val>
									</doml:extendedAttribute>
							</xsl:for-each>
						</xsl:if>
					</doml:extendedAttributes>
				</xsl:if>
				<xsl:if test="@isConstant='true'">
					<doml:value name="isConstant" default="true"/>
				</xsl:if>				
				<xsl:apply-templates/>
			</doml:column>	
		 </xsl:when>
		 <xsl:otherwise>
			<doml:foreignKey>
				<xsl:attribute name="name">
					<xsl:value-of select="@id"/>
				</xsl:attribute>
				<xsl:attribute name="id">
					<xsl:variable name="fullId" select="parent::table/@id"/>
					<xsl:call-template name="idShrunk">
						<xsl:with-param name="idList" select="$fullId"/>
					</xsl:call-template>-<xsl:value-of select="@id"/>
				</xsl:attribute>
				<xsl:attribute name="foreignTableId">
					<xsl:variable name="fullId" select="referenceObject/@reference"/>
					<xsl:call-template name="idShrunk">
						<xsl:with-param name="idList" select="$fullId"/>
					</xsl:call-template>		
				</xsl:attribute>
				
				<xsl:if test="referenceObject/@constraint!=''">
					<doml:extendedAttributes>
						<doml:extendedAttribute id="constraint">
							<xsl:attribute name="val">
								<xsl:value-of select="referenceObject/@constraint"/>
							</xsl:attribute>
						</doml:extendedAttribute>
					</doml:extendedAttributes>
				</xsl:if>
				<!--always create fkcolumn-->
				<!--xsl:if test="type/@canBeNull='true' or @usedForQuery!='' or @generateSecure!='' or @generateInsecure!='' or child::*[name(.)='error'] or child::*[name(.)='javadoc']"-->
					<doml:fkcolumn>
						<xsl:if test="type/@canBeNull='true'">
							<xsl:attribute name="canBeNull">
								<xsl:value-of select="type/@canBeNull"/>
							</xsl:attribute>
						</xsl:if>
						<xsl:if test="@usedForQuery!=''">
							<xsl:attribute name="usedForQuery">
								<xsl:value-of select="@usedForQuery"/>
							</xsl:attribute>
						</xsl:if>
												
						<xsl:attribute name="name">
							<xsl:value-of select="@id"/>
						</xsl:attribute>
						
						<xsl:attribute name="parentColumn">
							<!--xsl:value-of select="OId"/-->
							<xsl:text>OId</xsl:text>
						</xsl:attribute>
						
						<!--<xsl:if test="@generateSecure!='' or @generateInsecure!='' or type/@javaType!='' or child::*[name(.)='error'] or child::*[name(.)='javadoc']">-->
						<xsl:if test="@generateSecure!='' or @generateInsecure!='' or child::*[name(.)='error'] or child::*[name(.)='javadoc']">
							<doml:extendedAttributes>
									<xsl:if test="@generateSecure!=''">
											<doml:extendedAttribute id="generateSecure">
												<xsl:attribute name="val">
													<xsl:value-of select="@generateSecure"/>
												</xsl:attribute>
											</doml:extendedAttribute>
									</xsl:if>
									<xsl:if test="@generateInsecure!=''">
											<doml:extendedAttribute id="generateInsecure">
												<xsl:attribute name="val">
													<xsl:value-of select="@generateInsecure"/>
												</xsl:attribute>
											</doml:extendedAttribute>
									</xsl:if>
									<!--<xsl:if test="type/@javaType!=''">
											<doml:extendedAttribute id="javaType">
												<xsl:attribute name="value">
													<xsl:value-of select="type/@javaType"/>
												</xsl:attribute>
											</doml:extendedAttribute>
									</xsl:if>-->
									
									<xsl:if test="child::*[name(.)='error']">
										<xsl:for-each select="error|Error|ERROR">
												<doml:extendedAttribute id="error">
													<doml:val><xsl:value-of select="."/></doml:val>
												</doml:extendedAttribute>
										</xsl:for-each>
									</xsl:if>
									<xsl:if test="child::*[name(.)='javadoc']">
										<xsl:for-each select="javadoc|Javadoc|JAVADOC">
												<doml:extendedAttribute id="javadoc">
													<doml:val><xsl:value-of select="."/></doml:val>
												</doml:extendedAttribute>
										</xsl:for-each>
									</xsl:if>
							</doml:extendedAttributes>
						</xsl:if>
					</doml:fkcolumn>
				<xsl:apply-templates/>
			</doml:foreignKey>
		 </xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	
	<xsl:template match="initialValue|InitialValue|INITIALVALUE">
		<doml:value name="initialValue">
			<xsl:attribute name="default">
				<xsl:value-of select="."/>
			</xsl:attribute>
		</doml:value>
	</xsl:template>

	
    <xsl:template match="index|Index|INDEX">
		<doml:index>
			<xsl:attribute name="id">
				<xsl:variable name="fullId" select="parent::table/@id"/>
				<xsl:call-template name="idShrunk">
					<xsl:with-param name="idList" select="$fullId"/>
				</xsl:call-template>-<xsl:value-of select="@id"/>
			</xsl:attribute>
			<xsl:if test="@unique='true'">
				<xsl:attribute name="unique">
					<xsl:value-of select="@unique"/>
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="@clustered='true'">
				<xsl:attribute name="clustered">
					<xsl:value-of select="@clustered"/>
				</xsl:attribute>
			</xsl:if>
			
			<xsl:if test="child::*[name(.)='includeIndexColumn']">
				<doml:extendedAttributes>
					<doml:extendedAttribute id="includeIndexColumn">
						<doml:val>
							<xsl:for-each select="includeIndexColumn|IncludeIndexColumn|INCLUDEINDEXCOLUMN">
								<INCLUDE_INDEX_COLUMN>
									<xsl:attribute name="id">
										<xsl:value-of select="@id"/>
									</xsl:attribute>
								</INCLUDE_INDEX_COLUMN>
							</xsl:for-each>
						</doml:val>
					</doml:extendedAttribute>
				</doml:extendedAttributes>
			</xsl:if>			
			<xsl:apply-templates/>
		</doml:index>
	</xsl:template>

	
	 <xsl:template match="indexColumn|IndexColumn|INDEXCOLUMN">
		<xsl:element name="doml:include">
			<xsl:attribute name="column">
				<xsl:value-of select="@id"/>
			</xsl:attribute>
			<xsl:apply-templates/>
		</xsl:element>	
	</xsl:template>

	
	 <xsl:template match="statistics|Statistics|STATISTICS">
		<doml:index>
			<xsl:attribute name="id">
				<xsl:variable name="fullId" select="parent::table/@id"/>
				<xsl:call-template name="idShrunk">
					<xsl:with-param name="idList" select="$fullId"/>
				</xsl:call-template>-<xsl:value-of select="@id"/>
			</xsl:attribute>
			<xsl:if test="@unique='true'">
				<xsl:attribute name="unique">
					<xsl:value-of select="@unique"/>
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="@clustered='true'">
				<xsl:attribute name="clustered">
					<xsl:value-of select="@clustered"/>
				</xsl:attribute>
			</xsl:if>
		
			<doml:extendedAttributes>
				<doml:extendedAttribute id="isStatistics" val="true">
				</doml:extendedAttribute>
				<xsl:if test="not(@fullScan='')">
						<doml:extendedAttribute id="fullScan">
						<xsl:attribute name="val">
							<xsl:value-of select="@fullScan"/>
						</xsl:attribute>
						</doml:extendedAttribute>
				</xsl:if>
				<xsl:if test="not(@noRecompute='')">
						<doml:extendedAttribute id="noRecompute">
						<xsl:attribute name="val">
							<xsl:value-of select="@noRecompute"/>
						</xsl:attribute>
						</doml:extendedAttribute>
				</xsl:if>
				<xsl:if test="not(@sampleNo='')">
						<doml:extendedAttribute id="sampleNo">
						<xsl:attribute name="val">
							<xsl:value-of select="@sampleNo"/>
						</xsl:attribute>
						</doml:extendedAttribute>
				</xsl:if>
				<xsl:if test="not(@sampleType='')">
						<doml:extendedAttribute id="sampleType">
						<xsl:attribute name="val">
							<xsl:value-of select="@sampleType"/>
						</xsl:attribute>
						</doml:extendedAttribute>
				</xsl:if>
			</doml:extendedAttributes>			
			<xsl:apply-templates/>
		</doml:index>
	</xsl:template>

	
	 <xsl:template match="statisticsColumn|StatisticsColumn|STATISTICSCOLUMN">
		<doml:include>
			<xsl:attribute name="column">
				<xsl:value-of select="@id"/>
			</xsl:attribute>
			<xsl:apply-templates/>
		</doml:include>	
	</xsl:template>


	<xsl:template match="javadoc|error">
		<!-- do nothing - already handled -->
	</xsl:template>
	
	
<!-- ***************************************************** additional functions ***************************************************** -->
	
	<xsl:template name="idShrunk">
		<xsl:param name="idList"/>
		<xsl:choose>
			<xsl:when test="contains($idList,'.')">
					<xsl:call-template name="idShrunk">
						<xsl:with-param name="idList" select="substring-after($idList,'.')" />
					</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="$idList != ''">
					<xsl:value-of select="$idList"/>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
</xsl:template>

</xsl:stylesheet>
